@@ -1,14 +1,15 @@
 from abc import ABC, abstractmethod
 import logging
 import os
+from pathlib import Path
 import traceback
 
 from dotenv import load_dotenv
 
 from .types import Param
 
-# first thing first ... load dotenv file if any
-load_dotenv()
+# first thing first ... load dotenv file if any in the cwd
+load_dotenv(dotenv_path=Path.cwd() / ".env")
 
 LOGGER = logging.getLogger(__name__)
 
@@ -104,13 +105,32 @@ class ProcessCallback(Callback):
         else:
             try:
                 subprocess.run(self.to_cmd(param), shell=True, check=True)
-                engine.ack(param)
             except Exception as e:
-                engine.nack(param)
                 raise e
 
     def teardown(self, param, engine, exception):
-        return super().teardown(param, engine, exception)
+        if exception is None:
+            # everything is alright
+            self.teardown_ok(param, engine)
+            # we ack when ok hook has been successfully ran
+            engine.ack(param)
+            return
+        try:
+            raise exception
+        except KeyboardInterrupt:
+            engine.nack(param)
+            raise exception
+        except Exception:
+            pass
+        finally:
+            engine.nack(param)
+            self.teardown_ko(param, engine, exception)
+
+    def teardown_ok(self, param, engine):
+        pass
+
+    def teardown_ko(self, param, engine, exception):
+        pass
 
 
 class EntryPoint:
@@ -133,12 +153,18 @@ class EntryPoint:
             if param is None:
                 LOGGER.debug("No more params to handle, leaving")
                 break
-            self.cb.setup(param, self.engine)
             try:
+                # The user may want the setup to fail
+                # so we let the teardown decides what
+                # to do in this case
+                self.cb.setup(param, self.engine)
                 self.cb.process(param, self.engine)
             except Exception as e:
                 exception = e
-                traceback.print_exc()
+                LOGGER.error(
+                    "--> exception raised during execution" "(printing stack trace)"
+                )
+                LOGGER.error(traceback.format_exc())
             self.cb.teardown(param, self.engine, exception)
 
 
